from abc import ABC, abstractmethod
from threading import Thread

class Worker(ABC, Thread):
    @abstractmethod
    def configure(self, config):
        return NotImplemented

    @abstractmethod
    def work(self):
        return NotImplemented

    def run(self):
        self.work()

    def __init__(self, config):
        super().__init__()
        self.configure(config)

    def __del__(self):
        self.terminate()

__all__ = ['Worker']



from typing import Callable, Any

class _StateManager:
    _instance = None

    def __new__(cls):
        if not cls._instance:
            cls._instance = super(_StateManager, cls).__new__(cls)
        return cls._instance

    def __init__(self):
        self.state = {}
        self.subscriber = {}

    def set(self, key, value):
        self.state[key] = value
        for callback in self.subscriber.values():
            callback(value)

    def subscribe(self, key, callback):
        self.subscriber[key] = callback

    # TODO: unused
    def get(self, key):
        return self.state[key]

    # TODO: unused
    def delete(self, key):
        del self.state[key]

    # TODO: unused
    def clear(self):
        self.state = {}

class Pipeline:
    def __init__(self):
        self.manager = _StateManager()

    def publish(self, name: str, state) -> None:
        self.manager.set(name, state)

    def subscribe(self, name: str, callback: Callable[[Any], None]) -> None:
        self.manager.subscribe(name, callback)

__all__ = ['Pipeline']



from abc import ABC, abstractmethod
from threading import Thread

class Component(ABC, Thread):
    @abstractmethod
    def configure(self, config):
        return NotImplemented

    @abstractmethod
    def run(self):
        return NotImplemented

    def __del__(self):
        self.terminate()

class Generator(Component):
    def __init__(self):
        super().__init__()
        self.configure()

    def run(self):
        self.send()

    @abstractmethod
    def send(self):
        return NotImplemented

class Reactor(Component):
    def __init__(self):
        super().__init__()
        self.configure()

    def run(self):
        self.react()

    @abstractmethod
    def react(self):
        return NotImplemented


__all__ = ['Component', 'Generator', 'Reactor']
